package test.io;

import java.io.IOException;

import org.data2semantics.tools.graphs.Edge;
import org.data2semantics.tools.graphs.GML;
import org.data2semantics.tools.graphs.GML.LVertex;

import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.UndirectedGraph;
import edu.uci.ics.jung.graph.UndirectedSparseGraph;
import edu.uci.ics.jung.io.GraphIOException;

/**
 * Other than {@link GraphFormatGraphReader} and {@link RomeGraphFormatGraphReader} this class is based not
 * on own implementations, but here other code is used.
 * It is the function org.data2semantics.tools.graphs.{@link GML#read(java.io.File)} from data2semantics
 * <p>
 * Note:
 * The parsed graph has Stings as vertices and edges and is made undirected although the graph
 * being returned by org.data2semantics.tools.graphs.{@link GML#read(java.io.File)} might be directed.
 * Then the undirected graph gotten by assuming every directed edge to be undirected is returned
 * in {@link GmlFormatGraphReader#readGraph()}
 */
public class GmlFormatGraphReader extends MyGraphReader{

	@Override
	public UndirectedGraph<String, String> readGraph() throws GraphIOException {
		Graph<LVertex, Edge<String>> gmlGraph;
		try {
			gmlGraph = GML.read(graphFile);
		} catch (IOException e) {
			throw new GraphIOException();
		}
		//make this gml-graph a undirected graph with vertices and edges as strings.
		UndirectedGraph<String, String> graph = new UndirectedSparseGraph<String, String>();
		//add vertices
		for(LVertex lv: gmlGraph.getVertices()){
			String vName = lv.label;
			if(vName==null || vName=="" || vName==" "){
				vName = "v"+lv.id();
			}
			vertexNumberToVertex.put(lv.id(), vName);
			graph.addVertex(vName);
		}
		//add edges
		int edgeCounter = 0;
		for(Edge<String> e: gmlGraph.getEdges()){
			String eName = e.getLabel();
			if(eName==null || eName=="" || eName==" "){
				eName = "e"+edgeCounter;
			}
			graph.addEdge(eName,
					vertexNumberToVertex.get(gmlGraph.getEndpoints(e).getFirst().id()),
					vertexNumberToVertex.get(gmlGraph.getEndpoints(e).getSecond().id()));
			edgeCounter++;
		}
		return graph;
	}
}
